/*
 * DataTypes.c
 *
 * Converting between internal and external formats
 */

#include "butil.h"
#include "datatypes.h"
#include "roster.h"
#include "mystring.h"


void NameIn(short id,char *name)
{
	char str[13]; // 12 + 1 for '\0'
	
	// pad out string
	bzero(str,12);
	mystrcpy(str,name);
	
	// file data
	bcopy(str,gRoster[id].Name,12);
}

void NameOut(short id,char *name)
{	
	// read data
	mystrcpy(name,gRoster[id].Name);
}

void ForceIn(short id, short set)
{
	char marks;
	char mark = (1<<4);
	
	marks = gRoster[id].Marks;
	
	if (set) {
		marks |= mark;
	} else {
		marks &= ~mark;
	}
	gRoster[id].Marks = marks;
}

short ForceOut(short id)
{
	char marks;
	char mark = (1<<4);
	
	marks = gRoster[id].Marks;
	
	if (marks & mark) {
		return 1;
	} else {
		return 0;
	}
}

void FireIn(short id, short set)
{
	char marks;
	char mark = (1<<5);
	
	marks = gRoster[id].Marks;
	
	if (set) {
		marks |= mark;
	} else {
		marks &= ~mark;
	}
	gRoster[id].Marks = marks;
}

short FireOut(short id)
{
	char marks;
	char mark = (1<<5);
	
	marks = gRoster[id].Marks;
	
	if (marks & mark) {
		return 1;
	} else {
		return 0;
	}
}

void SnakeIn(short id, short set)
{
	char marks;
	char mark = (1<<6);
	
	marks = gRoster[id].Marks;
	
	if (set) {
		marks |= mark;
	} else {
		marks &= ~mark;
	}
	gRoster[id].Marks = marks;
}

short SnakeOut(short id)
{
	char marks;
	char mark = (1<<6);
	
	marks = gRoster[id].Marks;
	
	if (marks & mark) {
		return 1;
	} else {
		return 0;
	}
}

void KingsIn(short id, short set)
{
	char marks;
	char mark = (1<<7);
	
	marks = gRoster[id].Marks;
	
	if (set) {
		marks |= mark;
	} else {
		marks &= ~mark;
	}
	gRoster[id].Marks = marks;
}

short KingsOut(short id)
{
	char marks;
	char mark = (1<<7);
	
	marks = gRoster[id].Marks;
	
	if (marks & mark) {
		return 1;
	} else {
		return 0;
	}
}

void TorchesIn(short id,short torches)
{
	gRoster[id].Torches = (unsigned char)torches;
}

short TorchesOut(short id)
{
	return ((short)(gRoster[id].Torches));
}

void HealthIn(short id,short health)
{
	char h;
	
	// do conversion
	switch(health) {
	case kHealthGood:
		h='G';
		break;
	case kHealthPoisoned:
		h='P';
		break;
	case kHealthDead:
		h='D';
		break;
	case kHealthAshes:
		h='A';
		break;
	default:
		return; // DON'T FILE BAD DATA!!!
	}
	
	// file data
	gRoster[id].Health=h;
}

short HealthOut(short id)
{
	short h;

	// do conversion
	switch(gRoster[id].Health) {
	case 'G':
		h = kHealthGood;
		break;
	case 'P':
		h = kHealthPoisoned;
		break;
	case 'D':
		h = kHealthDead;
		break;
	case 'A':
		h = kHealthAshes;
		break;
	default:
		h = kHealthGood ;// why not?
	}
	
	return h;
}

void StrIn(short id, short Str) 
{
	gRoster[id].Str = (unsigned char)Str;
}

short StrOut(short id)
{
	return ((short)(gRoster[id].Str));
}

void DexIn(short id, short Dex) 
{
	gRoster[id].Dex = (unsigned char)Dex;
}

short DexOut(short id)
{
	return ((short)(gRoster[id].Dex));
}

void IntIn(short id, short Int) 
{
	gRoster[id].Int = (unsigned char)Int;
}

short IntOut(short id)
{
	return ((short)(gRoster[id].Int));
}

void WisIn(short id, short Wis) 
{
	gRoster[id].Wis = (unsigned char)Wis;
}

short WisOut(short id)
{
	return ((short)(gRoster[id].Wis));
}

void RaceIn(short id,short race)
{
	char r;
	
	// do conversion
	switch(race) {
	case kRaceHuman:
		r='H';
		break;
	case kRaceElf:
		r='E';
		break;
	case kRaceDwarf:
		r='D';
		break;
	case kRaceBobbit:
		r='B';
		break;
	case kRaceFuzzy:
		r='F';
		break;
	default:
		return; // DON'T FILE BAD DATA!!!
	}
	
	// file data
	gRoster[id].Race = r;
}

short RaceOut(short id)
{
	short r;

	// do conversion
	switch(gRoster[id].Race) {
	case 'H':
		r = kRaceHuman;
		break;
	case 'E':
		r = kRaceElf;
		break;
	case 'D':
		r = kRaceDwarf;
		break;
	case 'B':
		r = kRaceBobbit;
		break;
	case 'F':
		r = kRaceFuzzy;
		break;
	default:
		r = kRaceHuman; // why not?
	}
	
	return r;
}

void ClassIn(short id,short klass)
{
	char c;
	
	// do conversion
	switch(klass) {
	case kClassFighter:
		c = 'F';
		break;
	case kClassCleric:
		c = 'C';
		break;
	case kClassWizard:
		c = 'W';
		break;
	case kClassThief:
		c = 'T';
		break;
	case kClassPaladin:
		c = 'P';
		break;
	case kClassBarbarian:
		c = 'B';
		break;
	case kClassLark:
		c = 'L';
		break;
	case kClassIllusionist:
		c = 'I';
		break;
	case kClassDruid:
		c = 'D';
		break;
	case kClassAlchemist:
		c = 'A';
		break;
	case kClassRanger:
		c = 'R';
		break;
	default:
		return; // Again - we don't file bad data
	}
	
	// file data
	gRoster[id].Class = c;
}

short ClassOut(short id)
{
	short c;
	
	// do conversion
	switch(gRoster[id].Class) {
	case 'F':
		c = kClassFighter;
		break;
	case 'C':
		c = kClassCleric;
		break;
	case 'W':
		c = kClassWizard;
		break;
	case 'T':
		c = kClassThief;
		break;
	case 'P':
		c = kClassPaladin;
		break;
	case 'B':
		c = kClassBarbarian;
		break;
	case 'L':
		c = kClassLark;
		break;
	case 'I':
		c = kClassIllusionist;
		break;
	case 'D':
		c = kClassDruid;
		break;
	case 'A':
		c = kClassAlchemist;
		break;
	case 'R':
		c = kClassRanger;
		break;
	}
	
	return c;
}

void SexIn(short id,short sex)
{
	char s;
	
	// do conversion
	switch(sex) {
	case kSexFemale:
		s = 'F';
		break;
	case kSexMale:
		s = 'M';
		break;
	case kSexOther:
		s = 'O';
		break;
	default:
		return; // DON'T FILE BAD DATA!!!
	}
	
	// file data
	gRoster[id].Sex = s;
}

short SexOut(short id)
{
	short s;
	
	// do conversion
	switch(gRoster[id].Sex) {
	case 'F':
		s = kSexFemale;
		break;
	case 'M':
		s = kSexMale;
		break;
	case 'O':
		s = kSexOther;
		break;
	}
	
	return s;
}

void CurMPIn(short id, short CurMP) 
{
	gRoster[id].CurMP = (unsigned char)CurMP;
}

short CurMPOut(short id)
{
	return ((short)(gRoster[id].CurMP));
}

void CurHPIn(short id, unsigned short CurHP) 
{
	gRoster[id].CurHP = CurHP;

}

unsigned short CurHPOut(short id)
{
	return ((short)(gRoster[id].CurHP));
}

void MaxHPIn(short id, unsigned short MaxHP) 
{
	gRoster[id].MaxHP = MaxHP;
}

unsigned short MaxHPOut(short id)
{
	return ((short)(gRoster[id].MaxHP));
}

void XPIn(short id, unsigned short XP) 
{
	gRoster[id].XP1 = (unsigned char)(XP/100);
	gRoster[id].XP2 = (unsigned char)(XP%100);
}

unsigned short XPOut(short id)
{
	return ((unsigned short)((gRoster[id].XP1 * 100) + gRoster[id].XP2));
}

void FoodIn(short id, unsigned short Food) 
{
	gRoster[id].Food1 = (unsigned char)(Food/100);
	gRoster[id].Food2 = (unsigned char)(Food%100);
}

unsigned short FoodOut(short id)
{
	return ((short)((gRoster[id].Food1 * 100) + gRoster[id].Food2));
}

void GoldIn(short id, unsigned short Gold) 
{
	unsigned char *c;
	c = (unsigned char *)&Gold;
	gRoster[id].Gold1 = c[0];
	gRoster[id].Gold2 = c[1];
}

unsigned short GoldOut(short id)
{
	unsigned short *gold;
	gold = (unsigned short *)&(gRoster[id].Gold1);
	return (*gold);
}

void GemsIn(short id, short Gems) 
{
	gRoster[id].Gems = (unsigned char)Gems;
}

short GemsOut(short id)
{
	return ((short)(gRoster[id].Gems));
}

void KeysIn(short id, short Keys) 
{
	gRoster[id].Keys = (unsigned char)Keys;
}

short KeysOut(short id)
{
	return ((short)(gRoster[id].Keys));
}

void PowdersIn(short id, short Powders) 
{
	gRoster[id].Powders = (unsigned char)Powders;
}

short PowdersOut(short id)
{
	return ((short)(gRoster[id].Powders));
}

void EqArmIn(short id, short EqArm) 
{
	EqArm--;
	gRoster[id].EqArm = (unsigned char)EqArm;
}

short EqArmOut(short id)
{
	return ((short)(gRoster[id].EqArm)+1);
}

void EqWeapIn(short id, short EqWeap) 
{
	EqWeap--;
	gRoster[id].EqWeap = (unsigned char)EqWeap;
}

short EqWeapOut(short id)
{
	return ((short)(gRoster[id].EqWeap)+1);
}

void ArmIn(short id, short aID, unsigned char num)
{
	gRoster[id].Armors[aID]=num;
}

short ArmOut(short id, short aID)
{
	return ((short)(gRoster[id].Armors[aID]));
}

void WeapIn(short id, short wID, unsigned char num)
{
	gRoster[id].Weapons[wID]=num;
}

short WeapOut(short id, short wID)
{
	return ((short)(gRoster[id].Weapons[wID]));
}
