/*
 * DLOGUtil.h
 *
 * Dialog Utilities - Some routines useful in Dialog and Alert EventProcs.
 *
 */

#include "dlogutil.h"
#include "mystring.h"

#define kEnterKey		3
#define kBackspace		8
#define kTab			9
#define kReturnKey		13
#define kEscapeKey		27
#define kLeftKey		28
#define kRightKey		29
#define kUpKey			30
#define kDownKey		31
#define kDelete			0xFF


// oh no! software might break!
//StringHandle	TEScrpHandle = (StringHandle)0x0AB4;


/* IsDlgNumber
 * Determines if a character is valid input in a number-only dialog item.
 * In addition to verifying the number falls in the 0-9 range, it also
 * allows certain control characters; i.e., Return, Tab, arrows, etc.
 */
Boolean IsDlgNumber(char c)
{
	if (IsDlgControl(c) || IsNumber(c)) return true;
	
	return false;
}

Boolean IsDlgControl(char c)
{
	if ((c >= kEscapeKey) && (c <= kDownKey)) return true;
	if ((c == kReturnKey) || (c == kEnterKey) || (c == kDelete) 
		|| (c == kBackspace) || (c == kTab)) return true; 
	
	return false;
}

Boolean IsNumber(char c)
{
	if ((c >= '0') && (c<='9')) return true;
	
	return false;
}

Boolean IsPrintableChar(char c)
{
	if ((c > 32) && (c != 0xFF)) return true;
	return false;
}

/* MakeDLOGString
 * Given a DialogPtr, EventPtr and an item number, it returns what the string 
 * will be if the current event is processed. Should only be called from the 
 * dialog's event proc. If the input charater is a control character, the
 * routine will not alter the item's string.
 *
 * Oh ... I meant to say, it should only be called on TextEdit DLOG items.
 * 8/20/95
 */

void MakeDLOGString (DialogPtr d, EventRecord *e, short item, Str255 str)
{
	//short			*TEScrpLength = (short *)0x0AB0;
	short			TEScrpLength = TEGetScrapLength();//LMGetTEScrpLength();
	DialogRecord	*dr;
	TEHandle		teh;
	char			c;
	Str255			input, text;
	short 			selStart, // the character after which the ins. pt. lies
					selEnd;   // the character before which the ins. pt. lies
	short 			i;
	short			strIdx=0;
	
	short			iType;
	Handle			iHandle;
	Rect			iRect;
	
	// get the text string
	GetDItem(d,item,&iType,&iHandle,&iRect);
	GetIText(iHandle,text);

	// Set the input string
	c = (e->message & charCodeMask);
	// if it's a control character, return the dlog item text.
	if (IsDlgControl(c)) {
		PStrCpy(str,text);
		return;
	} else if (e->modifiers & cmdKey) {
		if  ((c == 'v') || (c == 'V')) {
			// if pasting ... get the pasted string.
			(void)TEFromScrap();
			HLock(iHandle);
			iHandle = TEScrapHandle();
			TEScrpLength = LMGetTEScrpLength();
			for (i=0; i< TEScrpLength/**TEScrpLength*/; i++) {
				input[i+1]=((unsigned char *)(*iHandle))[i];
			}
			input[0]=TEScrpLength;//*TEScrpLength;
			HUnlock(iHandle);
		} else { // if any other command stroke 
			PStrCpy(str,text);
			return;
		}
	} else {
		// else, set the input string equal to the new character
		input[0]=1;
		input[1]=(unsigned char) c;
	}
	 
	// get the selection point in the TERec
	dr = (DialogRecord *)d;
	teh = dr->textH;
	HLock((Handle)teh);
	selStart=(*teh)->selStart;
	selEnd=(*teh)->selEnd;
	HUnlock((Handle)teh);

	// generate output (str) string:
	//   copy the first bit of text:
	for (i=1; i<=selStart; i++) { 
		str[++strIdx]=text[i];
	}
	//   copy the input string:
	for (i=1; i<=input[0]; i++) {
		str[++strIdx]=input[i];
	}
	//   copy the last part of text:
	for (i=selEnd+1; i<=text[0]; i++) {
		str[++strIdx]=text[i];
	}
	//   lastly, set the length
	str[0] = strIdx;
}

/* AbortEvent
 * To be called from within a DLOG or ALRT EventProc. Beeps, sets event
 * to nullEvent, and returns false, thereby cancelling the event completely.
 */
Boolean	AbortEvent(EventRecord *e)
{
	SysBeep(3);
	e->what = nullEvent; 	// abort event
	return (false);
}
