/*
 * DataTypes.c
 *
 * Converting between internal and external formats.
 *
 * Version 1.1 8/95
 *
 * Copyright  1995 Michael D. Trent; all rights reserved
 */

/* Note : this requires the AQty and WQty arrays to be global ... damn it
 * mdt 8/16/95
 */

#include "butil.h"
#include "datatypes11.h"
#include "DSCH.h"
#include "mystring.h"
#include "ResIDs.h"
#include "roster11.h"
#include "u3cheat11.h"

void LoadControl(DialogPtr d, short item, short charId)
{
	DSCHEntry		dsch;
		
	unsigned short	retShort;
	Str255			retStr;
	
	short			iType,idx;
	Handle			iHandle;
	Rect			iRect;
	
	dsch=gDSCHList[item-3]; // shit - I can't use refCons, and I'm not sure if I can get to the DITL resource while a dialog is open ... 

	GetDItem(d,item,&iType,&iHandle,&iRect); // init iHandle here
	
	switch(dsch.dataType) {
	case DSCHNumber:
		switch(dsch.numericRep) {
		case kDSCHBinary:
			switch( dsch.size) {
			case 1:
				retShort = (unsigned short)gRoster[charId][dsch.offset];
				break;
			case 2:
				retShort = *((unsigned short *)&(gRoster[charId][dsch.offset]));
				break;
			default:
				DebugStr("\pLoadControl Error: Unknown data size!");
			}
			break;
		case kDSCHExoBinary:
			// Automatically assumes size = 2;
			retShort = ((unsigned short)((gRoster[charId][dsch.offset] * 100) +
							(gRoster[charId][dsch.offset+1]) ));
			break;
		default:
			DebugStr("\pLoadControl Error: Unknown Numeric Representation!");
		}
		
		NumToString((long)retShort,retStr);
		SetIText(iHandle,retStr);
		break; // DSCHNumber
		
	case DSCHString:
		C2PString((char *)&(gRoster[charId][dsch.offset]),retStr);
		SetIText(iHandle,retStr);
		break; //DSCHString
		
	case DSCHCategory:
		// each category is separately defined :
		
		switch (dsch.numericRep) {
		case kDSCHHealth:
			retShort = (unsigned short)LoadHealth(charId,dsch.offset);
			break;
		case kDSCHRace:
			retShort = (unsigned short)LoadRace(charId,dsch.offset);
			break;
		case kDSCHClass:
			retShort = (unsigned short)LoadClass(charId,dsch.offset);
			break;
		case kDSCHSex:
			retShort = (unsigned short)LoadSex(charId,dsch.offset);
			break;
		case kDSCHEqArm:
		case kDSCHEqWeap:
			retShort = gRoster[charId][dsch.offset] + 1;
			break;
		default:
			DebugStr("\pLoadControl Error: Bad Category Item!");
		}
		
		SetCtlValue((ControlHandle)iHandle,retShort);
		break; // DSCHCategory
	
	case DSCHBitMap:
		// this applies only to offset 14.
		retShort = (unsigned short)(gRoster[charId][dsch.offset] & (1 << dsch.numericRep)); // numericRep stores bit #
		if (retShort > 0) retShort = 255;
		
		SetCtlValue((ControlHandle)iHandle,retShort);
		break; //DSCHBitMap
		
	case DSCHByteMap:
		// this is a misleading case: it reads in a text field, and assigns it to
		// the proper byte in a byte map.  
		// Not only misleading - also ugly implementation! MDT 8/16/95
		for (idx=0; idx<dsch.size; idx++) {
			retShort = (unsigned short)gRoster[charId][dsch.offset+idx];
			if (item == kAQty) AQty[idx] = retShort;
			else if (item == kWQty) WQty[idx] = retShort;
		}
		break; // DSCHByteMap
		
	default:
		DebugStr("\pLoadControl Error: Unknown Data Type!");
	}
}

void SaveControl(DialogPtr d, short item, short charId)
{
	DSCHEntry		dsch;
		
	unsigned short	retShort;
	long			retLong;
	Str255			retStr;
	unsigned char	*retC;
	//unsigned char 	retS[13];
	
	short			iType,idx;
	Handle			iHandle;
	Rect			iRect;
	
	dsch=gDSCHList[item-3]; // shit - I can't use refCons, and I'm not sure if I can get to the DITL resource while a dialog is open ... 

	GetDItem(d,item,&iType,&iHandle,&iRect); // init iHandle here
	
	switch(dsch.dataType) {
	case DSCHNumber:
		GetIText(iHandle,retStr);
		StringToNum(retStr,&retLong);
		
		switch(dsch.numericRep) {
		case kDSCHBinary:
			switch( dsch.size) {
			case 1:
				gRoster[charId][dsch.offset]=(unsigned char)retLong;
				break;
			case 2:
				retShort = (unsigned short) retLong;
				retC = (unsigned char *)&retShort;
				gRoster[charId][dsch.offset]   = retC[0];
				gRoster[charId][dsch.offset+1] = retC[1];
				break;
			default:
				DebugStr("\pSaveControl Error: Unknown data size!");
			}
			break;
		case kDSCHExoBinary:
			// Automatically assumes size = 2;
			gRoster[charId][dsch.offset]   = (unsigned char)( retLong/100);
			gRoster[charId][dsch.offset+1] = (unsigned char) (retLong%100);
			break;
		default:
			DebugStr("\pSaveControl Error: Unknown Numeric Representation!");
		}
		break; // DSCHNumber
		
	case DSCHString:
		GetIText(iHandle,retStr);
		retC = (unsigned char *) NewPtr((Size)(dsch.size));
		bzero((char *) retC,dsch.size);
		P2CString(retStr,(char *)retC);
		bcopy((char *)retC,(char *)&(gRoster[charId][dsch.offset]),dsch.size);
		DisposePtr((Ptr)retC);
		break; //DSCHString
		
	case DSCHCategory:
		// each category is separately defined :
		retShort = GetCtlValue((ControlHandle)iHandle);
		
		switch (dsch.numericRep) {
		case kDSCHHealth:
			SaveHealth(charId,dsch.offset,retShort);
			break;
		case kDSCHRace:
			SaveRace(charId,dsch.offset,retShort);
			break;
		case kDSCHClass:
			SaveClass(charId,dsch.offset,retShort);
			break;
		case kDSCHSex:
			SaveSex(charId,dsch.offset,retShort);
			break;
		case kDSCHEqArm:
		case kDSCHEqWeap:
			gRoster[charId][dsch.offset] = retShort - 1;
			break;
		default:
			DebugStr("\pSaveControl Error: Bad Category Item!");
		}
		break; // DSCHCategory
	
	case DSCHBitMap:
		// this applies only to offset 14.
		if (GetCtlValue((ControlHandle)iHandle)) {
			gRoster[charId][dsch.offset] |= (1 << dsch.numericRep);
		} else {
			gRoster[charId][dsch.offset] &= ~(1 << dsch.numericRep);
		}
		break; //DSCHBitMap
		
	case DSCHByteMap:
		// this is a misleading case: it reads in a text field, and assigns it to
		// the proper byte in a byte map.  'idx' is the proper byte in the byte map,
		// incidentally (incase you were wondering what it was for)
		for (idx=0; idx<dsch.size; idx++) {
			if (item == kAQty) {
				gRoster[charId][dsch.offset+idx] = (unsigned char) AQty[idx];
			} else if (item == kWQty) {
				gRoster[charId][dsch.offset+idx] = (unsigned char) WQty[idx];
			}
		}

		break; // DSCHByteMap
		
	default:
		DebugStr("\pSaveControl Error: Unknown Data Type!");
	}
}

/* LoadRadioControl
 * This sets the radio control names - it's a highly specific case of LoadControl
 */
void LoadRadioControl(DialogPtr d, short charId)
{
	Str255 retStr;
	DSCHEntry dsch = gDSCHList[0]; // name
	
	short iType;
	Handle iHandle;
	Rect iRect;
	
	C2PString((char *)&(gRoster[charId][dsch.offset]),retStr);
	GetDItem(d,charId+4,&iType,&iHandle,&iRect); // +4 cause there are 3 buttons before the first radio.
	if (retStr[0] == 0) {
		SetCTitle((ControlHandle)iHandle,"\p<Empty>");
		HiliteControl((ControlHandle)iHandle,255); //(dim control)
	} else {
		SetCTitle((ControlHandle)iHandle,retStr);
	}
}

/**
 ** CATEGORY LISTS
 **/

void SaveHealth(short id,short offset,short health)
{
	char h;
	
	// do conversion
	switch(health) {
	case kHealthGood:
		h='G';
		break;
	case kHealthPoisoned:
		h='P';
		break;
	case kHealthDead:
		h='D';
		break;
	case kHealthAshes:
		h='A';
		break;
	default:
		return; // DON'T FILE BAD DATA!!!
	}
	
	// file data
	gRoster[id][offset]=h;
}

short LoadHealth(short id, short offset)
{
	short h;

	// do conversion
	switch(gRoster[id][offset]) {
	case 'G':
		h = kHealthGood;
		break;
	case 'P':
		h = kHealthPoisoned;
		break;
	case 'D':
		h = kHealthDead;
		break;
	case 'A':
		h = kHealthAshes;
		break;
	default:
		h = kHealthGood ;// why not?
	}
	
	return h;
}

void SaveRace(short id, short offset, short race)
{
	char r;
	
	// do conversion
	switch(race) {
	case kRaceHuman:
		r='H';
		break;
	case kRaceElf:
		r='E';
		break;
	case kRaceDwarf:
		r='D';
		break;
	case kRaceBobbit:
		r='B';
		break;
	case kRaceFuzzy:
		r='F';
		break;
	default:
		return; // DON'T FILE BAD DATA!!!
	}
	
	// file data
	gRoster[id][offset] = r;
}

short LoadRace(short id,short offset)
{
	short r;

	// do conversion
	switch(gRoster[id][offset]) {
	case 'H':
		r = kRaceHuman;
		break;
	case 'E':
		r = kRaceElf;
		break;
	case 'D':
		r = kRaceDwarf;
		break;
	case 'B':
		r = kRaceBobbit;
		break;
	case 'F':
		r = kRaceFuzzy;
		break;
	default:
		r = kRaceHuman; // why not?
	}
	
	return r;
}

void SaveClass(short id, short offset, short klass)
{
	char c;
	
	// do conversion
	switch(klass) {
	case kClassFighter:
		c = 'F';
		break;
	case kClassCleric:
		c = 'C';
		break;
	case kClassWizard:
		c = 'W';
		break;
	case kClassThief:
		c = 'T';
		break;
	case kClassPaladin:
		c = 'P';
		break;
	case kClassBarbarian:
		c = 'B';
		break;
	case kClassLark:
		c = 'L';
		break;
	case kClassIllusionist:
		c = 'I';
		break;
	case kClassDruid:
		c = 'D';
		break;
	case kClassAlchemist:
		c = 'A';
		break;
	case kClassRanger:
		c = 'R';
		break;
	default:
		return; // Again - we don't file bad data
	}
	
	// file data
	gRoster[id][offset] = c;
}

short LoadClass(short id,short offset)
{
	short c;
	
	// do conversion
	switch(gRoster[id][offset]) {
	case 'F':
		c = kClassFighter;
		break;
	case 'C':
		c = kClassCleric;
		break;
	case 'W':
		c = kClassWizard;
		break;
	case 'T':
		c = kClassThief;
		break;
	case 'P':
		c = kClassPaladin;
		break;
	case 'B':
		c = kClassBarbarian;
		break;
	case 'L':
		c = kClassLark;
		break;
	case 'I':
		c = kClassIllusionist;
		break;
	case 'D':
		c = kClassDruid;
		break;
	case 'A':
		c = kClassAlchemist;
		break;
	case 'R':
		c = kClassRanger;
		break;
	}
	
	return c;
}

void SaveSex(short id, short offset, short sex)
{
	char s;
	
	// do conversion
	switch(sex) {
	case kSexFemale:
		s = 'F';
		break;
	case kSexMale:
		s = 'M';
		break;
	case kSexOther:
		s = 'O';
		break;
	default:
		return; // DON'T FILE BAD DATA!!!
	}
	
	// file data
	gRoster[id][offset] = s;
}

short LoadSex(short id,short offset)
{
	short s;
	
	// do conversion
	switch(gRoster[id][offset]) {
	case 'F':
		s = kSexFemale;
		break;
	case 'M':
		s = kSexMale;
		break;
	case 'O':
		s = kSexOther;
		break;
	}
	
	return s;
}

/**
 ** MISC
 **/

long GetMaxItemSize(short item)
{
	DSCHEntry	dsch;
	long		retLong = 255;
	
	dsch=gDSCHList[item-3]; // shit - I can't use refCons, and I'm not sure if I can get to the DITL resource while a dialog is open ... 

	//assume dsch.dataType = Number
	if (dsch.size == 2) {
		if (dsch.numericRep == kDSCHBinary) retLong = 65535L;
		else if (dsch.numericRep == kDSCHExoBinary) retLong = 25599;
	}
	return retLong;
}

